﻿using System.Collections.Generic;
using System.IO;
using ArchiCop.Core;
using ArchiCop.InfoData;
using ArchiCop.ViewModel;
using Microsoft.Practices.Unity;

namespace ArchiCop.Controller
{
    public class ArchiCopController : IController
    {
        public ArchiCopController(IMainWindowViewModel mainWindowViewModel, IInfoRepository infoRepository)
        {
            string[] files = Directory.GetFiles(".", "*.xls");
            IEnumerable<ConfigInfo> configInfos = infoRepository.GetConfigInfos(files);

            foreach (ConfigInfo configInfo in configInfos)
            {
                var configInfoViewModel = new ConfigInfoViewModel(configInfo.DisplayName);

                foreach (DataSourceInfo dataSourceInfo in configInfo.DataSources)
                {
                    var dataSourceInfoViewModel = new DataSourceInfoViewModel(dataSourceInfo.DisplayName,
                                                                              ToDataSourceType(
                                                                                  dataSourceInfo.LoadEngine.EngineType));

                    dataSourceInfoViewModel.Graph = dataSourceInfo.GetGraph();

                    foreach (GraphInfo graphInfo in configInfo.Graphs)
                    {
                        if (graphInfo.DataSource.Name == dataSourceInfo.Name)
                        {
                            var graphInfoViewModel = new GraphInfoViewModel(graphInfo.DisplayName);

                            graphInfoViewModel.Graph = graphInfo.GetGraph();

                            dataSourceInfoViewModel.Graphs.Add(graphInfoViewModel);
                        }
                    }

                    configInfoViewModel.DataSources.Add(dataSourceInfoViewModel);
                }

                mainWindowViewModel.Configurations.Add(configInfoViewModel);
            }

        }

        private static DataSourceType ToDataSourceType(LoadEngineType loadEngineType)
        {
            switch (loadEngineType)
            {
                case LoadEngineType.VisualStudio:
                    return DataSourceType.VisualStudio;
                default:
                    return DataSourceType.Data;
            }
        }

        [InjectionMethod]
        public void Initialize(IMainWindowViewModel mainWindowViewModel,
                                  ArchiCopSolutionViewModel archiCopSolutionViewModel)
        {
            mainWindowViewModel.ControlPanelCommands.Add(archiCopSolutionViewModel);
        }
    }
}