﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Windows.Data;
using ArchiCop.Core;
using ArchiCop.Properties;
using Microsoft.Practices.Unity;
using MvvmFoundation.Wpf;

namespace ArchiCop.ViewModel
{
    public class ArchiCopSolutionViewModel : CommandViewModel
    {
        private readonly ObservableCollection<GraphCommandViewModel> _cachedCommands =
            new ObservableCollection<GraphCommandViewModel>();

        private readonly IMainWindowViewModel _mainWindowViewModel;

        [InjectionConstructor]
        public ArchiCopSolutionViewModel(IMainWindowViewModel mainWindowViewModel)
            : base(Resources.ArchiCopSolutionViewModel_DisplayName)
        {
            _mainWindowViewModel = mainWindowViewModel;
            Commands = new ObservableCollection<GraphCommandViewModel>();

            Files = mainWindowViewModel.Configurations;

            ICollectionView filesView = CollectionViewSource.GetDefaultView(Files);
            filesView.CurrentChanged += FilesCurrentChanged;

            foreach (ConfigInfoViewModel configInfo in mainWindowViewModel.Configurations)
            {
                foreach (DataSourceInfoViewModel dataSource in configInfo.DataSources)
                {
                    var graphCommandViewModelType = GraphCommandViewModelType.Datasource;
                    if (dataSource.DataSourceType == DataSourceType.VisualStudio)
                    {
                        graphCommandViewModelType=GraphCommandViewModelType.VisualStudioDatasource;
                    }

                    _cachedCommands.Add(CreateGraphCommandViewModel(dataSource.Graph.DisplayName,
                                                                            graphCommandViewModelType,
                                                                            configInfo.DisplayName, dataSource.Graph));                    
                    foreach (GraphInfoViewModel graph in dataSource.Graphs)
                    {
                        _cachedCommands.Add(CreateGraphCommandViewModel(graph.DisplayName, GraphCommandViewModelType.Graph,
                                                      configInfo.DisplayName, graph.Graph));                          
                    }
                }
            }
        }

        private GraphCommandViewModel CreateGraphCommandViewModel(string displayName,GraphCommandViewModelType graphCommandViewModelType,string tag, ArchiCopGraph<ArchiCopVertex> graph)
        {
            var graphCommandViewModel = new GraphCommandViewModel(displayName, graphCommandViewModelType, tag);

            graphCommandViewModel.ShowEdgesCommand =
                new RelayCommand<object>(param => ShowGraphEdgesView(graph));
            graphCommandViewModel.ShowGraphCommand =
                new RelayCommand<object>(param => ShowGraphView(graph));
            graphCommandViewModel.ShowVerticesCommand =
                new RelayCommand<object>(param => ShowGraphVerticesView(graph));

            return graphCommandViewModel;
        }

        public ObservableCollection<GraphCommandViewModel> Commands { get; private set; }

        public ObservableCollection<ConfigInfoViewModel> Files { get; set; }

        private void FilesCurrentChanged(object sender, EventArgs e)
        {
            var tag = ((ICollectionView) sender).CurrentItem as ConfigInfoViewModel;

            Commands.Clear();


            foreach (
                GraphCommandViewModel commandViewModel in _cachedCommands.Where(item => item.Tag == tag.DisplayName))
            {
                Commands.Add(commandViewModel);
            }
        }


        private void ShowGraphView(ArchiCopGraph<ArchiCopVertex> graph)
        {
            var workspace =
                _mainWindowViewModel.Workspaces.Where(vm => vm is GraphViewModel).
                                     FirstOrDefault(vm => vm.Tag == "Graph" + graph.DisplayName) as
                GraphViewModel;

            if (workspace == null)
            {
                workspace = new GraphViewModel(graph, "Graph", "Graph" + graph.DisplayName);
                _mainWindowViewModel.Workspaces.Add(workspace);
            }

            _mainWindowViewModel.SetActiveWorkspace(workspace);
        }

        private void ShowGraphVerticesView(ArchiCopGraph<ArchiCopVertex> graph)
        {
            var workspace =
                _mainWindowViewModel.Workspaces.Where(vm => vm is GraphVerticesViewModel).
                                     FirstOrDefault(vm => vm.Tag == "Vertices" + graph.DisplayName) as
                GraphVerticesViewModel;

            if (workspace == null)
            {
                workspace = new GraphVerticesViewModel(graph, "Vertices", "Vertices" + graph.DisplayName);
                _mainWindowViewModel.Workspaces.Add(workspace);
            }

            _mainWindowViewModel.SetActiveWorkspace(workspace);
        }

        private void ShowGraphEdgesView(ArchiCopGraph<ArchiCopVertex> graph)
        {
            var workspace =
                _mainWindowViewModel.Workspaces.Where(vm => vm is GraphEdgesViewModel).
                                     FirstOrDefault(vm => vm.Tag == "Edges" + graph.DisplayName) as
                GraphEdgesViewModel;

            if (workspace == null)
            {
                workspace = new GraphEdgesViewModel(graph, "Edges", "Edges" + graph.DisplayName);
                _mainWindowViewModel.Workspaces.Add(workspace);
            }

            _mainWindowViewModel.SetActiveWorkspace(workspace);
        }
    }
}