﻿using System.Collections.Generic;
using System.Linq;
using NUnit.Framework;

namespace ArchiCop.InfoData
{    
    [TestFixture]
    public class ExcelInfoRepositoryTests
    {
        private readonly IEnumerable<DataSourceInfo> _dataSources =
            new ExcelInfoRepository().GetConfigInfos(@"InfoData\Sample.xls").First().DataSources;

        private readonly IEnumerable<GraphInfo> _graphs =
            new ExcelInfoRepository().GetConfigInfos(@"InfoData\Sample.xls").First().Graphs;

        [Test]
        public void HasCorrectNumberOfGraphInfos()
        {
            //
            Assert.AreEqual(_graphs.Count(), 4);
        }

        [Test]
        public void HasCorrectNumberOfDataSourceInfos()
        {
            //
            Assert.AreEqual(_dataSources.Count(), 2);
        }

        [Test]
        public void GraphInfosHaveCorrectDisplayNames()
        {
            //
            string[] source = _graphs.Select(item => item.DisplayName).ToArray();
            var target = new[]
                {
                    "ArchiCop1",
                    "ArchiCop2",
                    "Graph Test 1",
                    "Graph Test 2"
                };
            Assert.IsTrue(source.Intersect(target).Any());
        }

        [Test]
        public void DataSourceInfosHaveCorrectDisplayNames()
        {
            //
            string[] source = _dataSources.Select(item => item.DisplayName).ToArray();
            var target = new[]
                {
                    "archicop",
                    "test1",
                };
            Assert.IsTrue(source.Intersect(target).Any());
        }

        [Test]
        public void GraphInfosHaveCorrectLoadEngines_EngineName()
        {
            //
            string[] source = _graphs.Select(item => item.DataSource.LoadEngine.EngineName).ToArray();
            var target = new[]
                {
                    "ArchiCop.Core.VisualStudioProjectLoadEngine,ArchiCopCore",
                    "ArchiCop.Core.VisualStudioProjectLoadEngine,ArchiCopCore",
                    "ArchiCop.Core.LoadEngineExcel,ArchiCopCore",
                    "ArchiCop.Core.LoadEngineExcel,ArchiCopCore"
                };
            Assert.IsTrue(source.Intersect(target).Any());
        }

        [Test]
        public void DataSourceInfosHaveCorrectLoadEngines_EngineName()
        {
            //
            string[] source = _dataSources.Select(item => item.LoadEngine.EngineName).ToArray();
            var target = new[]
                {
                    "ArchiCop.Core.VisualStudioProjectLoadEngine,ArchiCopCore",
                    "ArchiCop.Core.LoadEngineExcel,ArchiCopCore",
                };
            Assert.IsTrue(source.Intersect(target).Any());
        }

        [Test]
        public void GraphInfosHaveCorrectLoadEngines_Arg1()
        {
            //
            string[] source = _graphs.Select(item => item.DataSource.LoadEngine.Arg1).ToArray();
            var target = new[]
                {
                    @"..\..\..",
                    @"..\..\..",
                    "Sample.xls",
                    "Sample.xls"
                };
            Assert.IsTrue(source.Intersect(target).Any());
        }

        [Test]
        public void DataSourceInfosHaveCorrectLoadEngines_Arg1()
        {
            //
            string[] source = _dataSources.Select(item => item.LoadEngine.Arg1).ToArray();
            var target = new[]
                {
                    @"..\..\..",
                    "Sample.xls"
                };
            Assert.IsTrue(source.Intersect(target).Any());
        }

        [Test]
        public void GraphInfosHaveCorrectLoadEngines_Arg2()
        {
            //
            string[] source = _graphs.Select(item => item.DataSource.LoadEngine.Arg2).ToArray();
            var target = new[]
                {
                    null,
                    null,
                    "DataTest$",
                    "DataTest$"
                };
            Assert.IsTrue(source.Intersect(target).Any());
        }

        [Test]
        public void DataSourceInfosHaveCorrectLoadEngines_Arg2()
        {
            //
            string[] source = _dataSources.Select(item => item.LoadEngine.Arg2).ToArray();
            var target = new[]
                {
                    null,
                    "DataTest$"
                };
            Assert.IsTrue(source.Intersect(target).Any());
        }
    }
}