using System.Collections.Generic;
using System.Linq;
using ArchiCop.InfoData;

namespace ArchiCop.Core
{
    public static class ArchiCopGraphEngine
    {
        public static ArchiCopGraph<ArchiCopVertex> GetGraph(this DataSourceInfo dataSourceInfo)
        {
            ArchiCopGraph<ArchiCopVertex> graph = GetGraph(dataSourceInfo.LoadEngine);

            graph.DisplayName = dataSourceInfo.DisplayName;

            return graph;
        }

        public static ArchiCopGraph<ArchiCopVertex> GetGraph(this GraphInfo graphInfo)
        {
            IEnumerable<VertexRegexRule> rules = graphInfo.Rules.Select(
                item =>
                new VertexRegexRule
                    {
                        Pattern = item.RulePattern,
                        Value = item.RuleValue
                    });

            ArchiCopGraph<ArchiCopVertex> graph = GetGraph(graphInfo.DataSource.LoadEngine, rules.ToArray());

            graph.DisplayName = graphInfo.DisplayName;

            return graph;
        }

       

        private static ArchiCopGraph<ArchiCopVertex> GetGraph(LoadEngineInfo loadEngineInfo,
                                                              params VertexRegexRule[] vertexRegexRules)
        {
            var loadEngine = (ILoadEngine) loadEngineInfo.CreateLoadEngine();
            ArchiCopGraph<ArchiCopVertex> graph = loadEngine.LoadGraph();

            if (vertexRegexRules.Any())
            {
                IEnumerable<ArchiCopEdge<ArchiCopVertex>> edges = new EdgeEngineRegex().ConvertEdges(graph.Edges,
                                                                                                     vertexRegexRules);
                graph = new ArchiCopGraph<ArchiCopVertex>();
                graph.AddVerticesAndEdgeRange(edges);
            }

            return graph;
        }
    }
}