﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using ArchiCop.Core;
using ArchiCop.InfoData;
using ArchiCop.VisualStudioData;

namespace ArchiMate
{
    public partial class MainForm : Form
    {
        private readonly IVisualStudioProjectRepository _projectRepository;
        private readonly IVisualStudioSolutionRepository _solutionRepository;
        private ArchiCopGraph<VisualStudioProject> _graph;
        private ArchiCopGraph<VisualStudioProject> _graphCached;

        public MainForm(IVisualStudioProjectRepository projectRepository, IVisualStudioSolutionRepository solutionRepository)
        {
            InitializeComponent();
            _projectRepository = projectRepository;
            _solutionRepository = solutionRepository;

            IInfoRepository infoRepository = new ExcelInfoRepository();
            string[] files = Directory.GetFiles(Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location), "*.xls", SearchOption.TopDirectoryOnly);

            var dataSourceInfos = new List<DataSourceInfo>();
            foreach (List<DataSourceInfo> list in infoRepository.GetConfigInfos(files).Select(item => item.DataSources))
            {
                dataSourceInfos.AddRange(list);
            }

            dataSourceInfos =
                dataSourceInfos.Where(item => item.LoadEngine.EngineType == LoadEngineType.VisualStudio).ToList();

            comboBox1.DataSource = dataSourceInfos;
            comboBox1.DisplayMember = "DisplayName";
            comboBox1.ValueMember = "Name";
            
        }

       
        private void BindGrids()
        {
            IEnumerable<ArchiCopEdge<VisualStudioProject>> edges = _graph.Edges;

            if (!checkBox1.Checked)
            {
                edges = edges.Where(item => item.Source.ProjectType != ".csproj").ToList();
                edges = edges.Where(item => item.Target.ProjectType != ".csproj").ToList();
            }
            if (!checkBox2.Checked)
            {
                edges = edges.Where(item => item.Source.ProjectType != ".fsproj").ToList();
                edges = edges.Where(item => item.Target.ProjectType != ".fsproj").ToList();
            }
            if (!checkBox3.Checked)
            {
                edges = edges.Where(item => item.Source.ProjectType != ".vbproj").ToList();
                edges = edges.Where(item => item.Target.ProjectType != ".vbproj").ToList();
            }

            edges = edges.Where(item => Regex.IsMatch(item.Source.ProjectName, textBox2.Text))
                         .Where(item => Regex.IsMatch(item.Target.ProjectName, textBox3.Text)).ToList();

            _graphCached = edges.GetGraph();

            dataGridView2.DataSource = _graphCached.Edges.Select(item =>
                                                                 new
                                                                     {
                                                                         Source = item.Source.ProjectName,
                                                                         Target = item.Target.ProjectName
                                                                     })
                                                   .ToList();
            tabPage2.Text = "References (" + edges.Count() + ")";

            IEnumerable<VisualStudioProject> vertices = _graphCached.Vertices;

            dataGridView1.DataSource =
                vertices.OrderBy(item => item.ProjectName)
                        .Select(
                            item =>
                            new
                                {
                                    item.ProjectName,
                                    item.ProjectType,
                                    item.TargetFrameworkVersion,
                                    item.OutputType,
                                    item.AssemblyName,
                                    item.RootNamespace,
                                    item.ProjectPath,
                                    item.ProjectTypeGuids,
                                    item.ProjectTypes
                                })
                        .ToList();
            tabPage1.Text = "Projects (" + vertices.Count() + ")";
        }

        private void button2_Click(object sender, EventArgs e)
        {
            BindGrids();
        }

        private void button3_Click(object sender, EventArgs e)
        {
            _solutionRepository.CreateNewSolution(_graphCached, textBox1.Text + @"\Sample.Sln");
        }

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            var dataSourceInfo = (DataSourceInfo) comboBox1.SelectedItem;
            var loadEngine = (VisualStudioProjectLoadEngine)dataSourceInfo.LoadEngine.CreateLoadEngine();

            _graph = _projectRepository.GetProjects(loadEngine.ProjectsFiles).GetGraph();

            textBox1.Text = dataSourceInfo.LoadEngine.Arg1;

            BindGrids();
        }
        
    }
}