﻿using System;
using System.Data.Services;
using System.Data.Services.Common;
using System.Xml.Serialization;

namespace TestApp
{
    internal class Program
    {
        private static void Main(string[] args)
        {

            Type serviceType = typeof (VisualStudioProjectService);
            Uri baseAddress = new Uri("http://localhost:8000/VisualStudioProjectService");
            Uri[] baseAddresses = new Uri[] {baseAddress};

            DataServiceHost host = new DataServiceHost(serviceType, baseAddresses);

            host.Open();

            Console.WriteLine(
                "Navigate to the following URI to see the service.");
            Console.WriteLine(baseAddress);
            Console.WriteLine("Press any key to exit...");
            Console.ReadKey();

            host.Close();

        }
    }

    [DataServiceKeyAttribute("Id")]
    [IgnoreProperties("PropertyGroups", "ItemGroups")]
    [XmlRoot(Namespace = "http://schemas.microsoft.com/developer/msbuild/2003")]
    public class Project
    {
        public string Id { get; set; }

        [XmlAttribute]
        public string ToolsVersion { get; set; }

        [XmlAttribute]
        public string DefaultTargets { get; set; }

        [XmlElement("PropertyGroup")]
        public PropertyGroup[] PropertyGroups { get; set; }

        [XmlElement("ItemGroup")]
        public ItemGroup[] ItemGroups { get; set; }
    }

    [DataServiceKeyAttribute("Id")]
    public class PropertyGroup
    {
        public string Id { get; set; }

        [XmlAttribute]
        public string Condition { get; set; }

        [XmlElement]
        public Configuration Configuration { get; set; }

        [XmlElement]
        public Platform Platform { get; set; }

        public string ProjectGuid { get; set; }

        public string OutputType { get; set; }

        public string RootNamespace { get; set; }

        public string AssemblyName { get; set; }

        public string TargetFrameworkVersion { get; set; }

        public string ProjectTypeGuids { get; set; }

        public int WarningLevel { get; set; }

        public string PlatformTarget { get; set; }

        public bool DebugSymbols { get; set; }

        public string DebugType { get; set; }

        public bool Optimize { get; set; }

        public string OutputPath { get; set; }

        public string DefineConstants { get; set; }

        public string ErrorReport { get; set; }

        public bool Prefer32Bit { get; set; }

        public string DocumentationFile { get; set; }
    }

    [DataServiceKeyAttribute("Id")]
    public class Configuration
    {
        public string Id { get; set; }

        [XmlTextAttribute]
        public string Text { get; set; }

        [XmlAttribute]
        public string Condition { get; set; }
    }

    [DataServiceKeyAttribute("Id")]
    public class Platform
    {
        public string Id { get; set; }

        [XmlTextAttribute]
        public string Text { get; set; }

        [XmlAttribute]
        public string Condition { get; set; }
    }

    [DataServiceKeyAttribute("Id")]
    public class ItemGroup
    {
        public string Id { get; set; }

        [XmlElement("Reference")]
        public Reference[] Referenses { get; set; }

        [XmlElement("ProjectReference")]
        public ProjectReference[] ProjectReferenses { get; set; }

        [XmlElement("Compile")]
        public Compile[] Compiles { get; set; }

        [XmlElement("Page")]
        public Page[] Pages { get; set; }

        [XmlElement("Resource")]
        public Resource[] Resources { get; set; }

        [XmlElement("None")]
        public None[] Nones { get; set; }
    }

    [DataServiceKeyAttribute("Id")]
    public class Reference
    {
        public string Id { get; set; }

        [XmlAttribute]
        public string Include { get; set; }

        public string SpecificVersion { get; set; }

        public string HintPath { get; set; }
    }

    [DataServiceKeyAttribute("Id")]
    public class ProjectReference
    {
        public string Id { get; set; }

        [XmlAttribute]
        public string Include { get; set; }

        public string Project { get; set; }

        public string Name { get; set; }
    }

    [DataServiceKeyAttribute("Include")]
    public class Compile
    {
        [XmlAttribute]
        public string Include { get; set; }

        [XmlElement]
        public string DependentUpon { get; set; }

        [XmlElement]
        public string SubType { get; set; }

        [XmlElement]
        public string AutoGen { get; set; }

        [XmlElement]
        public string DesignTime { get; set; }

        [XmlElement]
        public string DesignTimeSharedInput { get; set; }

    }

    [DataServiceKeyAttribute("Include")]
    public class Page
    {
        [XmlAttribute]
        public string Include { get; set; }

        [XmlElement]
        public string Generator { get; set; }

        [XmlElement]
        public string SubType { get; set; }
    }

    [DataServiceKeyAttribute("Include")]
    public class Resource
    {
        [XmlAttribute]
        public string Include { get; set; }
    }

    [DataServiceKeyAttribute("Include")]
    public class None
    {
        [XmlAttribute]
        public string Include { get; set; }
    }



}
