﻿using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Xml.Serialization;

namespace TestApp
{
    public class VisualStudioProjectContext
    {
        List<Project> _projects = new List<Project>();

        public VisualStudioProjectContext()
        {
            string[] files = Directory.GetFiles(@"C:\Roche.DP.C4000\IM\Main\Units", "*.csproj", SearchOption.AllDirectories);

            foreach (string file in files)
            {
                var serializer = new XmlSerializer(typeof(Project));
                var fs = new FileStream(file, FileMode.Open, FileAccess.Read, FileShare.Read);
                var project = (Project)serializer.Deserialize(fs);
                project.Id = file;
                foreach (PropertyGroup propertyGroup in project.PropertyGroups)
                {
                    propertyGroup.Id = file;
                }
                foreach (ItemGroup itemGroup in project.ItemGroups)
                {
                    itemGroup.Id = file;
                    if (itemGroup.ProjectReferenses!=null)
                    {
                        foreach (ProjectReference projectReferense in itemGroup.ProjectReferenses)
                        {
                            projectReferense.Id = file;
                        }
                    }
                    if (itemGroup.Referenses!=null)
                    {
                        foreach (Reference referense in itemGroup.Referenses)
                        {
                            referense.Id = file;
                        }
                    }                    
                }
                _projects.Add(project);
                fs.Close();
            }
        }

        public IQueryable<Project> Projects
        {
            get
            {
                return  _projects.AsQueryable<Project>();
            }
        }

        public IQueryable<PropertyGroup> PropertyGroups
        {
            get
            {
                var propertyGroups = new List<PropertyGroup>();


                foreach (Project project in _projects)
                {
                    propertyGroups.AddRange(project.PropertyGroups);
                }

                return propertyGroups.AsQueryable();
            }
        }

        public IQueryable<Reference> References
        {
            get
            {
                var references = new List<Reference>();

                foreach (Project project in _projects)
                {
                    foreach (var itemGroup in project.ItemGroups)
                    {
                        if (itemGroup.Referenses!=null)
                        {
                            references.AddRange(itemGroup.Referenses);                            
                        }
                    }

                }

                return references.AsQueryable();
            }
        }

        public IQueryable<ProjectReference> ProjectReferences
        {
            get
            {
                var references = new List<ProjectReference>();

                foreach (Project project in _projects)
                {
                    foreach (var itemGroup in project.ItemGroups)
                    {
                        if (itemGroup.ProjectReferenses!=null)
                        {
                            references.AddRange(itemGroup.ProjectReferenses);                            
                        }
                    }

                }

                return references.AsQueryable();
            }
        }

    }
}